from typing import Any
from abc import ABC, abstractmethod
from asyncio import AbstractEventLoop
import json
from time import time

from ..nats_streaming import nats
from .types import Subjects


class AbstractPublisher(ABC):
    @property
    @abstractmethod
    def subject(self) -> Subjects:
        raise NotImplementedError()

    @property
    @abstractmethod
    def client_id(self) -> str:
        raise NotImplementedError()

    @abstractmethod
    def publish(self, data: Any, loop: AbstractEventLoop):
        pass

    async def on_publish(self, data: Any, loop: AbstractEventLoop):
        await nats.connect(
            'sea-web-services',
            f'{self.client_id}_{int(time() * 1000)}',
            'http://nats-cluster-ip-service:4222',
            loop,
        )
        await nats.sc.publish(self.subject, json.dumps(data).encode('utf-8'))
        print(f'Event published to <{self.subject}>')

        await nats.close()



import os


class BaseConfig:
    """Base configuration."""
    DEBUG = False
    TESTING = False
    SECRET_KEY = os.environ.get('SECRET_KEY')
    SQLALCHEMY_TRACK_MODIFICATIONS = False

    @staticmethod
    def init_app(app):
        pass


class DevelopmentConfig(BaseConfig):
    """Development configuration."""
    DEBUG = True
    SQLALCHEMY_DATABASE_URI = 'sqlite://'


class TestingConfig(BaseConfig):
    """Testing configuration."""
    TESTING = True
    SQLALCHEMY_DATABASE_URI = 'sqlite://'


class ProductionConfig(BaseConfig):
    """Production configuration."""
    DEBUG = False
    SQLALCHEMY_DATABASE_URI = 'sqlite://'


config = {
    'development': DevelopmentConfig,
    'testing': TestingConfig,
    'production': ProductionConfig,
    'default': DevelopmentConfig,
}



from typing import Optional
from asyncio import AbstractEventLoop
from nats.aio.client import Client as Nats
from stan.aio.client import Client as Stan


class NatsStreaming:
    nc: Optional[Nats] = None
    sc: Optional[Stan] = None

    async def connect(self, cluster_id: str, client_id: str, url: str, loop):
        # Use borrowed connection for NATS then mount NATS Streaming client on top.
        self.nc = Nats()
        await self.nc.connect(servers=[url], io_loop=loop)

        # Start session with NATS Streaming cluster.
        self.sc = Stan()
        await self.sc.connect(cluster_id, client_id, nats=self.nc)

        print('Connected to NATS Streaming server!')

    async def close(self):
        await self.sc.close()
        await self.nc.close()

    async def shutdown(self, loop: AbstractEventLoop):
        try:
            await nats.close()
            print('NATS connection closed')
        except:
            pass
        finally:
            loop.stop()


nats = NatsStreaming()



from enum import Enum
from typing import TypedDict
import strawberry


class Subjects(Enum):
    TargetCreated = 'target:created'
    AntisenseSeriesCreated = 'antisenseSeries:created'


class TargetCreatedData(TypedDict):
    organism_id: int
    id: strawberry.ID
    custom: bool
    rna_id: str
    spliced: bool
    symbol: str


class AntisenseSeriesCreatedData(TypedDict):
    target_id: int
    id: strawberry.ID
    length: int



import os
import operator
from typing import Optional, TypedDict
from flask import current_app, g, request
from werkzeug.exceptions import Unauthorized
from strawberry.flask.views import GraphQLView
from strawberry.extensions import ExecutionContext, Extension


class UserPayload(TypedDict):
    id: int
    username: str
    email: str


class MyContext(TypedDict):
    current_user: Optional[UserPayload]


class MyGraphQLView(GraphQLView):
    def get_context(self) -> MyContext:
        payload: Optional[UserPayload] = g.current_user

        return {'current_user': payload}


class AuthExtension(Extension):
    def on_request_start(self, *, execution_context: ExecutionContext):
        current_user = execution_context.context.get('current_user')

        if (
            current_user  # Valid login credentials provided
            or execution_context.query
            == 'query __ApolloGetServiceDefinition__ { _service { sdl } }'  # Gateway initialization
            or (os.environ.get('FLASK_ENV') == 'development' and request.headers.get('origin')
                == f"http://localhost:{os.environ.get('NODE_PORT')}")  # GraphiQL request
            or current_app.config['TESTING']  # Test environment
        ):
            return
        else:
            raise Unauthorized()


operator_keys = {
    '': operator.eq,
    '>': operator.ge,
    '<': operator.le,
}



from .app import create_app, db
from .config import BaseConfig
from .lib.types import Node, Connection, organism_mapping, InputWithOrganism, MutationResponse
from .lib.util_sqlalchemy import ResourceMixin, ResourceMixinWithVersion, RefseqMixin, ExternalResourceMixin, sort_query
from .lib.util_strawberry import MyGraphQLView, AuthExtension, operator_keys
from .events import AbstractPublisher, Subjects, TargetCreatedData, AntisenseSeriesCreatedData
